@@ -167,6 +167,14 @@ class W_LibObject(W_Root):
                 w_ctfnptr = w_ct.unwrap_as_fnptr(self.ffi)
                 w_result = W_CData(self.space, ptr, w_ctfnptr)
                 #
+                #
+            elif op == cffi_opcode.OP_EXTERN_PYTHON:
+                # for reading 'lib.bar' where bar is declared
+                # as an extern "Python"
+                w_ct = realize_c_type.realize_c_type(
+                    self.ffi, self.ctx.c_types, getarg(g.c_type_op))
+                ptr = lltype.direct_fieldptr(g, 'c_size_or_direct_fn')
+                w_result = w_ct.convert_to_object(rffi.cast(rffi.CCHARP, ptr))
             else:
                 raise oefmt(space.w_NotImplementedError,
                             "in lib_build_attr: op=%d", op)
@@ -186,6 +194,8 @@ class W_LibObject(W_Root):
                     return self.dir1(ignore_global_vars=True)
                 if is_getattr and attr == '__dict__':
                     return self.full_dict_copy()
+                if is_getattr and attr == '__class__':
+                    return self.space.type(self)
                 if is_getattr and attr == '__name__':
                     return self.descr_repr()
                 raise oefmt(self.space.w_AttributeError,
