@@ -38,7 +38,9 @@ def result_type(space, __args__):
             dtypes_w.append(dtype)
     return find_result_type(space, arrays_w, dtypes_w)
 
-
+@jit.look_inside_iff(lambda space, arrays_w, dtypes_w:
+    jit.loop_unrolling_heuristic(arrays_w, len(arrays_w)) and
+    jit.loop_unrolling_heuristic(dtypes_w, len(dtypes_w)))
 def find_result_type(space, arrays_w, dtypes_w):
     # equivalent to PyArray_ResultType
     if len(arrays_w) == 1 and not dtypes_w:
@@ -89,6 +91,9 @@ simple_kind_ordering = {
     NPY.STRINGLTR: 3, NPY.STRINGLTR2: 3,
     UnicodeType.kind: 3, VoidType.kind: 3, ObjectType.kind: 3}
 
+# this is safe to unroll since it'll only be seen if we look inside
+# the find_result_type
+@jit.unroll_safe
 def _use_min_scalar(arrays_w, dtypes_w):
     """Helper for find_result_type()"""
     if not arrays_w:
