@@ -1,4 +1,4 @@
-import weakref
+import weakref, sys
 
 from rpython.rlib import jit, objectmodel, debug, rerased
 from rpython.rlib.rarithmetic import intmask, r_uint
@@ -6,11 +6,17 @@ from rpython.rlib.rarithmetic import intmask, r_uint
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.objspace.std.dictmultiobject import (
     W_DictMultiObject, DictStrategy, ObjectDictStrategy, BaseKeyIterator,
-    BaseValueIterator, BaseItemIterator, _never_equal_to_string
+    BaseValueIterator, BaseItemIterator, _never_equal_to_string,
+    W_DictObject,
 )
 from pypy.objspace.std.typeobject import MutableCell
 
 
+erase_item, unerase_item = rerased.new_erasing_pair("mapdict storage item")
+erase_map,  unerase_map = rerased.new_erasing_pair("map")
+erase_list, unerase_list = rerased.new_erasing_pair("mapdict storage list")
+
+
 # ____________________________________________________________
 # attribute shapes
 
@@ -19,6 +25,7 @@ NUM_DIGITS_POW2 = 1 << NUM_DIGITS
 # note: we use "x * NUM_DIGITS_POW2" instead of "x << NUM_DIGITS" because
 # we want to propagate knowledge that the result cannot be negative
 
+
 class AbstractAttribute(object):
     _immutable_fields_ = ['terminator']
     cache_attrs = None
@@ -29,10 +36,10 @@ class AbstractAttribute(object):
         assert isinstance(terminator, Terminator)
         self.terminator = terminator
 
-    def read(self, obj, selector):
-        attr = self.find_map_attr(selector)
+    def read(self, obj, name, index):
+        attr = self.find_map_attr(name, index)
         if attr is None:
-            return self.terminator._read_terminator(obj, selector)
+            return self.terminator._read_terminator(obj, name, index)
         if (
             jit.isconstant(attr.storageindex) and
             jit.isconstant(obj) and
@@ -46,39 +53,26 @@ class AbstractAttribute(object):
     def _pure_mapdict_read_storage(self, obj, storageindex):
         return obj._mapdict_read_storage(storageindex)
 
-    def write(self, obj, selector, w_value):
-        attr = self.find_map_attr(selector)
+    def write(self, obj, name, index, w_value):
+        attr = self.find_map_attr(name, index)
         if attr is None:
-            return self.terminator._write_terminator(obj, selector, w_value)
+            return self.terminator._write_terminator(obj, name, index, w_value)
         if not attr.ever_mutated:
             attr.ever_mutated = True
         obj._mapdict_write_storage(attr.storageindex, w_value)
         return True
 
-    def delete(self, obj, selector):
+    def delete(self, obj, name, index):
         pass
 
-    def find_map_attr(self, selector):
-        if jit.we_are_jitted():
-            # hack for the jit:
-            # the _find_map_attr method is pure too, but its argument is never
-            # constant, because it is always a new tuple
-            return self._find_map_attr_jit_pure(selector[0], selector[1])
-        else:
-            return self._find_map_attr_indirection(selector)
-
     @jit.elidable
-    def _find_map_attr_jit_pure(self, name, index):
-        return self._find_map_attr_indirection((name, index))
-
-    @jit.dont_look_inside
-    def _find_map_attr_indirection(self, selector):
+    def find_map_attr(self, name, index):
         if (self.space.config.objspace.std.withmethodcache):
-            return self._find_map_attr_cache(selector)
-        return self._find_map_attr(selector)
+            return self._find_map_attr_cache(name, index)
+        return self._find_map_attr(name, index)
 
     @jit.dont_look_inside
-    def _find_map_attr_cache(self, selector):
+    def _find_map_attr_cache(self, name, index):
         space = self.space
         cache = space.fromcache(MapAttrCache)
         SHIFT2 = r_uint.BITS - space.config.objspace.std.methodcachesizeexp
@@ -86,31 +80,36 @@ class AbstractAttribute(object):
         attrs_as_int = objectmodel.current_object_addr_as_int(self)
         # ^^^Note: see comment in typeobject.py for
         # _pure_lookup_where_with_method_cache()
-        hash_selector = objectmodel.compute_hash(selector)
+
+        # unrolled hash computation for 2-tuple
+        c1 = 0x345678
+        c2 = 1000003
+        hash_name = objectmodel.compute_hash(name)
+        hash_selector = intmask((c2 * ((c2 * c1) ^ hash_name)) ^ index)
         product = intmask(attrs_as_int * hash_selector)
         attr_hash = (r_uint(product) ^ (r_uint(product) << SHIFT1)) >> SHIFT2
         # ^^^Note2: same comment too
         cached_attr = cache.attrs[attr_hash]
         if cached_attr is self:
-            cached_selector = cache.selectors[attr_hash]
-            if cached_selector == selector:
+            cached_name = cache.names[attr_hash]
+            cached_index = cache.indexes[attr_hash]
+            if cached_name == name and cached_index == index:
                 attr = cache.cached_attrs[attr_hash]
                 if space.config.objspace.std.withmethodcachecounter:
-                    name = selector[0]
                     cache.hits[name] = cache.hits.get(name, 0) + 1
                 return attr
-        attr = self._find_map_attr(selector)
+        attr = self._find_map_attr(name, index)
         cache.attrs[attr_hash] = self
-        cache.selectors[attr_hash] = selector
+        cache.names[attr_hash] = name
+        cache.indexes[attr_hash] = index
         cache.cached_attrs[attr_hash] = attr
         if space.config.objspace.std.withmethodcachecounter:
-            name = selector[0]
             cache.misses[name] = cache.misses.get(name, 0) + 1
         return attr
 
-    def _find_map_attr(self, selector):
+    def _find_map_attr(self, name, index):
         while isinstance(self, PlainAttribute):
-            if selector == self.selector:
+            if index == self.index and name == self.name:
                 return self
             self = self.back
         return None
@@ -136,40 +135,133 @@ class AbstractAttribute(object):
 
     @jit.elidable
     def _get_new_attr(self, name, index):
-        selector = name, index
         cache = self.cache_attrs
         if cache is None:
             cache = self.cache_attrs = {}
-        attr = cache.get(selector, None)
+        attr = cache.get((name, index), None)
         if attr is None:
-            attr = PlainAttribute(selector, self)
-            cache[selector] = attr
+            attr = PlainAttribute(name, index, self)
+            cache[name, index] = attr
         return attr
 
-    @jit.look_inside_iff(lambda self, obj, selector, w_value:
-            jit.isconstant(self) and
-            jit.isconstant(selector[0]) and
-            jit.isconstant(selector[1]))
-    def add_attr(self, obj, selector, w_value):
-        # grumble, jit needs this
-        attr = self._get_new_attr(selector[0], selector[1])
-        oldattr = obj._get_mapdict_map()
+    @jit.elidable
+    def _get_cache_attr(self, name, index):
+        key = name, index
+        # this method is not actually elidable, but it's fine anyway
+        if self.cache_attrs is not None:
+            return self.cache_attrs.get(key, None)
+        return None
+
+    def add_attr(self, obj, name, index, w_value):
+        self._reorder_and_add(obj, name, index, w_value)
         if not jit.we_are_jitted():
+            oldattr = self
+            attr = obj._get_mapdict_map()
             size_est = (oldattr._size_estimate + attr.size_estimate()
                                                - oldattr.size_estimate())
             assert size_est >= (oldattr.length() * NUM_DIGITS_POW2)
             oldattr._size_estimate = size_est
-        if attr.length() > obj._mapdict_storage_length():
-            # note that attr.size_estimate() is always at least attr.length()
-            new_storage = [None] * attr.size_estimate()
+
+    def _add_attr_without_reordering(self, obj, name, index, w_value):
+        attr = self._get_new_attr(name, index)
+        attr._switch_map_and_write_storage(obj, w_value)
+
+    @jit.unroll_safe
+    def _switch_map_and_write_storage(self, obj, w_value):
+        if self.length() > obj._mapdict_storage_length():
+            # note that self.size_estimate() is always at least self.length()
+            new_storage = [None] * self.size_estimate()
             for i in range(obj._mapdict_storage_length()):
                 new_storage[i] = obj._mapdict_read_storage(i)
-            obj._set_mapdict_storage_and_map(new_storage, attr)
+            obj._set_mapdict_storage_and_map(new_storage, self)
 
         # the order is important here: first change the map, then the storage,
         # for the benefit of the special subclasses
-        obj._set_mapdict_map(attr)
-        obj._mapdict_write_storage(attr.storageindex, w_value)
+        obj._set_mapdict_map(self)
+        obj._mapdict_write_storage(self.storageindex, w_value)
+
+
+    @jit.elidable
+    def _find_branch_to_move_into(self, name, index):
+        # walk up the map chain to find an ancestor with lower order that
+        # already has the current name as a child inserted
+        current_order = sys.maxint
+        number_to_readd = 0
+        current = self
+        key = (name, index)
+        while True:
+            attr = None
+            if current.cache_attrs is not None:
+                attr = current.cache_attrs.get(key, None)
+            if attr is None or attr.order > current_order:
+                # we reached the top, so we didn't find it anywhere,
+                # just add it to the top attribute
+                if not isinstance(current, PlainAttribute):
+                    return 0, self._get_new_attr(name, index)
+
+            else:
+                return number_to_readd, attr
+            # if not found try parent
+            number_to_readd += 1
+            current_order = current.order
+            current = current.back
+
+    @jit.look_inside_iff(lambda self, obj, name, index, w_value:
+            jit.isconstant(self) and
+            jit.isconstant(name) and
+            jit.isconstant(index))
+    def _reorder_and_add(self, obj, name, index, w_value):
+        # the idea is as follows: the subtrees of any map are ordered by
+        # insertion.  the invariant is that subtrees that are inserted later
+        # must not contain the name of the attribute of any earlier inserted
+        # attribute anywhere
+        #                              m______
+        #         inserted first      / \ ... \   further attributes
+        #           attrname a      0/  1\    n\
+        #                           m  a must not appear here anywhere
+        #
+        # when inserting a new attribute in an object we check whether any
+        # parent of lower order has seen that attribute yet. if yes, we follow
+        # that branch. if not, we normally append that attribute. When we
+        # follow a prior branch, we necessarily remove some attributes to be
+        # able to do that. They need to be re-added, which has to follow the
+        # reordering procedure recusively.
+
+        # we store the to-be-readded attribute in the stack, with the map and
+        # the value paired up those are lazily initialized to a list large
+        # enough to store all current attributes
+        stack = None
+        stack_index = 0
+        while True:
+            current = self
+            number_to_readd = 0
+            number_to_readd, attr = self._find_branch_to_move_into(name, index)
+            # we found the attributes further up, need to save the
+            # previous values of the attributes we passed
+            if number_to_readd:
+                if stack is None:
+                    stack = [erase_map(None)] * (self.length() * 2)
+                current = self
+                for i in range(number_to_readd):
+                    assert isinstance(current, PlainAttribute)
+                    w_self_value = obj._mapdict_read_storage(
+                            current.storageindex)
+                    stack[stack_index] = erase_map(current)
+                    stack[stack_index + 1] = erase_item(w_self_value)
+                    stack_index += 2
+                    current = current.back
+            attr._switch_map_and_write_storage(obj, w_value)
+
+            if not stack_index:
+                return
+
+            # readd the current top of the stack
+            stack_index -= 2
+            next_map = unerase_map(stack[stack_index])
+            w_value = unerase_item(stack[stack_index + 1])
+            name = next_map.name
+            index = next_map.index
+            self = obj._get_mapdict_map()
 
     def materialize_r_dict(self, space, obj, dict_w):
         raise NotImplementedError("abstract base class")
@@ -188,11 +280,11 @@ class Terminator(AbstractAttribute):
         AbstractAttribute.__init__(self, space, self)
         self.w_cls = w_cls
 
-    def _read_terminator(self, obj, selector):
+    def _read_terminator(self, obj, name, index):
         return None
 
-    def _write_terminator(self, obj, selector, w_value):
-        obj._get_mapdict_map().add_attr(obj, selector, w_value)
+    def _write_terminator(self, obj, name, index, w_value):
+        obj._get_mapdict_map().add_attr(obj, name, index, w_value)
         return True
 
     def copy(self, obj):
@@ -230,40 +322,40 @@ class DictTerminator(Terminator):
 
 
 class NoDictTerminator(Terminator):
-    def _write_terminator(self, obj, selector, w_value):
-        if selector[1] == DICT:
+    def _write_terminator(self, obj, name, index, w_value):
+        if index == DICT:
             return False
-        return Terminator._write_terminator(self, obj, selector, w_value)
+        return Terminator._write_terminator(self, obj, name, index, w_value)
 
 
 class DevolvedDictTerminator(Terminator):
-    def _read_terminator(self, obj, selector):
-        if selector[1] == DICT:
+    def _read_terminator(self, obj, name, index):
+        if index == DICT:
             space = self.space
             w_dict = obj.getdict(space)
-            return space.finditem_str(w_dict, selector[0])
-        return Terminator._read_terminator(self, obj, selector)
+            return space.finditem_str(w_dict, name)
+        return Terminator._read_terminator(self, obj, name, index)
 
-    def _write_terminator(self, obj, selector, w_value):
-        if selector[1] == DICT:
+    def _write_terminator(self, obj, name, index, w_value):
+        if index == DICT:
             space = self.space
             w_dict = obj.getdict(space)
-            space.setitem_str(w_dict, selector[0], w_value)
+            space.setitem_str(w_dict, name, w_value)
             return True
-        return Terminator._write_terminator(self, obj, selector, w_value)
+        return Terminator._write_terminator(self, obj, name, index, w_value)
 
-    def delete(self, obj, selector):
+    def delete(self, obj, name, index):
         from pypy.interpreter.error import OperationError
-        if selector[1] == DICT:
+        if index == DICT:
             space = self.space
             w_dict = obj.getdict(space)
             try:
-                space.delitem(w_dict, space.wrap(selector[0]))
+                space.delitem(w_dict, space.wrap(name))
             except OperationError, ex:
                 if not ex.match(space, space.w_KeyError):
                     raise
             return Terminator.copy(self, obj)
-        return Terminator.delete(self, obj, selector)
+        return Terminator.delete(self, obj, name, index)
 
     def remove_dict_entries(self, obj):
         assert 0, "should be unreachable"
@@ -275,27 +367,29 @@ class DevolvedDictTerminator(Terminator):
         return Terminator.set_terminator(self, obj, terminator)
 
 class PlainAttribute(AbstractAttribute):
-    _immutable_fields_ = ['selector', 'storageindex', 'back', 'ever_mutated?']
+    _immutable_fields_ = ['name', 'index', 'storageindex', 'back', 'ever_mutated?', 'order']
 
-    def __init__(self, selector, back):
+    def __init__(self, name, index, back):
         AbstractAttribute.__init__(self, back.space, back.terminator)
-        self.selector = selector
+        self.name = name
+        self.index = index
         self.storageindex = back.length()
         self.back = back
         self._size_estimate = self.length() * NUM_DIGITS_POW2
         self.ever_mutated = False
+        self.order = len(back.cache_attrs) if back.cache_attrs else 0
 
     def _copy_attr(self, obj, new_obj):
-        w_value = self.read(obj, self.selector)
-        new_obj._get_mapdict_map().add_attr(new_obj, self.selector, w_value)
+        w_value = self.read(obj, self.name, self.index)
+        new_obj._get_mapdict_map().add_attr(new_obj, self.name, self.index, w_value)
 
-    def delete(self, obj, selector):
-        if selector == self.selector:
+    def delete(self, obj, name, index):
+        if index == self.index and name == self.name:
             # ok, attribute is deleted
             if not self.ever_mutated:
                 self.ever_mutated = True
             return self.back.copy(obj)
-        new_obj = self.back.delete(obj, selector)
+        new_obj = self.back.delete(obj, name, index)
         if new_obj is not None:
             self._copy_attr(obj, new_obj)
         return new_obj
@@ -314,14 +408,14 @@ class PlainAttribute(AbstractAttribute):
         return new_obj
 
     def search(self, attrtype):
-        if self.selector[1] == attrtype:
+        if self.index == attrtype:
             return self
         return self.back.search(attrtype)
 
     def materialize_r_dict(self, space, obj, dict_w):
         new_obj = self.back.materialize_r_dict(space, obj, dict_w)
-        if self.selector[1] == DICT:
-            w_attr = space.wrap(self.selector[0])
+        if self.index == DICT:
+            w_attr = space.wrap(self.name)
             dict_w[w_attr] = obj._mapdict_read_storage(self.storageindex)
         else:
             self._copy_attr(obj, new_obj)
@@ -329,12 +423,12 @@ class PlainAttribute(AbstractAttribute):
 
     def remove_dict_entries(self, obj):
         new_obj = self.back.remove_dict_entries(obj)
-        if self.selector[1] != DICT:
+        if self.index != DICT:
             self._copy_attr(obj, new_obj)
         return new_obj
 
     def __repr__(self):
-        return "<PlainAttribute %s %s %r>" % (self.selector, self.storageindex, self.back)
+        return "<PlainAttribute %s %s %s %r>" % (self.name, self.index, self.storageindex, self.back)
 
 def _become(w_obj, new_obj):
     # this is like the _become method, really, but we cannot use that due to
@@ -346,8 +440,8 @@ class MapAttrCache(object):
         assert space.config.objspace.std.withmethodcache
         SIZE = 1 << space.config.objspace.std.methodcachesizeexp
         self.attrs = [None] * SIZE
-        self._empty_selector = (None, INVALID)
-        self.selectors = [self._empty_selector] * SIZE
+        self.names = [None] * SIZE
+        self.indexes = [INVALID] * SIZE
         self.cached_attrs = [None] * SIZE
         if space.config.objspace.std.withmethodcachecounter:
             self.hits = {}
@@ -356,8 +450,9 @@ class MapAttrCache(object):
     def clear(self):
         for i in range(len(self.attrs)):
             self.attrs[i] = None
-        for i in range(len(self.selectors)):
-            self.selectors[i] = self._empty_selector
+        for i in range(len(self.names)):
+            self.names[i] = None
+            self.indexes[i] = INVALID
         for i in range(len(self.cached_attrs)):
             self.cached_attrs[i] = None
 
@@ -387,28 +482,28 @@ class BaseMapdictObject:
     # objspace interface
 
     def getdictvalue(self, space, attrname):
-        return self._get_mapdict_map().read(self, (attrname, DICT))
+        return self._get_mapdict_map().read(self, attrname, DICT)
 
     def setdictvalue(self, space, attrname, w_value):
-        return self._get_mapdict_map().write(self, (attrname, DICT), w_value)
+        return self._get_mapdict_map().write(self, attrname, DICT, w_value)
 
     def deldictvalue(self, space, attrname):
-        new_obj = self._get_mapdict_map().delete(self, (attrname, DICT))
+        new_obj = self._get_mapdict_map().delete(self, attrname, DICT)
         if new_obj is None:
             return False
         self._become(new_obj)
         return True
 
     def getdict(self, space):
-        w_dict = self._get_mapdict_map().read(self, ("dict", SPECIAL))
+        w_dict = self._get_mapdict_map().read(self, "dict", SPECIAL)
         if w_dict is not None:
             assert isinstance(w_dict, W_DictMultiObject)
             return w_dict
 
         strategy = space.fromcache(MapDictStrategy)
         storage = strategy.erase(self)
-        w_dict = W_DictMultiObject(space, strategy, storage)
-        flag = self._get_mapdict_map().write(self, ("dict", SPECIAL), w_dict)
+        w_dict = W_DictObject(space, strategy, storage)
+        flag = self._get_mapdict_map().write(self, "dict", SPECIAL, w_dict)
         assert flag
         return w_dict
 
@@ -417,9 +512,14 @@ class BaseMapdictObject:
         w_dict = check_new_dictionary(space, w_dict)
         w_olddict = self.getdict(space)
         assert isinstance(w_dict, W_DictMultiObject)
-        if type(w_olddict.strategy) is not ObjectDictStrategy:
-            w_olddict.strategy.switch_to_object_strategy(w_olddict)
-        flag = self._get_mapdict_map().write(self, ("dict", SPECIAL), w_dict)
+        # The old dict has got 'self' as dstorage, but we are about to
+        # change self's ("dict", SPECIAL) attribute to point to the
+        # new dict.  If the old dict was using the MapDictStrategy, we
+        # have to force it now: otherwise it would remain an empty
+        # shell that continues to delegate to 'self'.
+        if type(w_olddict.get_strategy()) is MapDictStrategy:
+            w_olddict.get_strategy().switch_to_object_strategy(w_olddict)
+        flag = self._get_mapdict_map().write(self, "dict", SPECIAL, w_dict)
         assert flag
 
     def getclass(self, space):
@@ -437,16 +537,16 @@ class BaseMapdictObject:
         self._init_empty(w_subtype.terminator)
 
     def getslotvalue(self, slotindex):
-        key = ("slot", SLOTS_STARTING_FROM + slotindex)
-        return self._get_mapdict_map().read(self, key)
+        index = SLOTS_STARTING_FROM + slotindex
+        return self._get_mapdict_map().read(self, "slot", index)
 
     def setslotvalue(self, slotindex, w_value):
-        key = ("slot", SLOTS_STARTING_FROM + slotindex)
-        self._get_mapdict_map().write(self, key, w_value)
+        index = SLOTS_STARTING_FROM + slotindex
+        self._get_mapdict_map().write(self, "slot", index, w_value)
 
     def delslotvalue(self, slotindex):
-        key = ("slot", SLOTS_STARTING_FROM + slotindex)
-        new_obj = self._get_mapdict_map().delete(self, key)
+        index = SLOTS_STARTING_FROM + slotindex
+        new_obj = self._get_mapdict_map().delete(self, "slot", index)
         if new_obj is None:
             return False
         self._become(new_obj)
@@ -456,7 +556,7 @@ class BaseMapdictObject:
 
     def getweakref(self):
         from pypy.module._weakref.interp__weakref import WeakrefLifeline
-        lifeline = self._get_mapdict_map().read(self, ("weakref", SPECIAL))
+        lifeline = self._get_mapdict_map().read(self, "weakref", SPECIAL)
         if lifeline is None:
             return None
         assert isinstance(lifeline, WeakrefLifeline)
@@ -466,11 +566,11 @@ class BaseMapdictObject:
     def setweakref(self, space, weakreflifeline):
         from pypy.module._weakref.interp__weakref import WeakrefLifeline
         assert isinstance(weakreflifeline, WeakrefLifeline)
-        self._get_mapdict_map().write(self, ("weakref", SPECIAL), weakreflifeline)
+        self._get_mapdict_map().write(self, "weakref", SPECIAL, weakreflifeline)
     setweakref._cannot_really_call_random_things_ = True
 
     def delweakref(self):
-        self._get_mapdict_map().write(self, ("weakref", SPECIAL), None)
+        self._get_mapdict_map().write(self, "weakref", SPECIAL, None)
     delweakref._cannot_really_call_random_things_ = True
 
 class ObjectMixin(object):
@@ -531,9 +631,6 @@ def memo_get_subclass_of_correct_size(space, supercls):
 memo_get_subclass_of_correct_size._annspecialcase_ = "specialize:memo"
 _subclass_cache = {}
 
-erase_item, unerase_item = rerased.new_erasing_pair("mapdict storage item")
-erase_list, unerase_list = rerased.new_erasing_pair("mapdict storage list")
-
 def _make_subclass_size_n(supercls, n):
     from rpython.rlib import unroll
     rangen = unroll.unrolling_iterable(range(n))
@@ -636,7 +733,7 @@ class MapDictStrategy(DictStrategy):
         w_obj = self.unerase(w_dict.dstorage)
         strategy = self.space.fromcache(ObjectDictStrategy)
         dict_w = strategy.unerase(strategy.get_empty_storage())
-        w_dict.strategy = strategy
+        w_dict.set_strategy(strategy)
         w_dict.dstorage = strategy.erase(dict_w)
         assert w_obj.getdict(self.space) is w_dict or w_obj._get_mapdict_map().terminator.w_cls is None
         materialize_r_dict(self.space, w_obj, dict_w)
@@ -715,7 +812,7 @@ class MapDictStrategy(DictStrategy):
         curr = self.unerase(w_dict.dstorage)._get_mapdict_map().search(DICT)
         if curr is None:
             raise KeyError
-        key = curr.selector[0]
+        key = curr.name
         w_value = self.getitem_str(w_dict, key)
         w_key = self.space.wrap(key)
         self.delitem(w_dict, w_key)
@@ -745,14 +842,14 @@ class MapDictIteratorKeys(BaseKeyIterator):
 
     def next_key_entry(self):
         implementation = self.dictimplementation
-        assert isinstance(implementation.strategy, MapDictStrategy)
+        assert isinstance(implementation.get_strategy(), MapDictStrategy)
         if self.orig_map is not self.w_obj._get_mapdict_map():
             return None
         if self.curr_map:
             curr_map = self.curr_map.search(DICT)
             if curr_map:
                 self.curr_map = curr_map.back
-                attr = curr_map.selector[0]
+                attr = curr_map.name
                 w_attr = self.space.wrap(attr)
                 return w_attr
         return None
@@ -767,14 +864,14 @@ class MapDictIteratorValues(BaseValueIterator):
 
     def next_value_entry(self):
         implementation = self.dictimplementation
-        assert isinstance(implementation.strategy, MapDictStrategy)
+        assert isinstance(implementation.get_strategy(), MapDictStrategy)
         if self.orig_map is not self.w_obj._get_mapdict_map():
             return None
         if self.curr_map:
             curr_map = self.curr_map.search(DICT)
             if curr_map:
                 self.curr_map = curr_map.back
-                attr = curr_map.selector[0]
+                attr = curr_map.name
                 return self.w_obj.getdictvalue(self.space, attr)
         return None
 
@@ -788,14 +885,14 @@ class MapDictIteratorItems(BaseItemIterator):
 
     def next_item_entry(self):
         implementation = self.dictimplementation
-        assert isinstance(implementation.strategy, MapDictStrategy)
+        assert isinstance(implementation.get_strategy(), MapDictStrategy)
         if self.orig_map is not self.w_obj._get_mapdict_map():
             return None, None
         if self.curr_map:
             curr_map = self.curr_map.search(DICT)
             if curr_map:
                 self.curr_map = curr_map.back
-                attr = curr_map.selector[0]
+                attr = curr_map.name
                 w_attr = self.space.wrap(attr)
                 return w_attr, self.w_obj.getdictvalue(self.space, attr)
         return None, None
@@ -878,9 +975,9 @@ def LOAD_ATTR_slowpath(pycode, w_obj, nameindex, map):
             _, w_descr = w_type._pure_lookup_where_possibly_with_method_cache(
                 name, version_tag)
             #
-            selector = ("", INVALID)
+            attrname, index = ("", INVALID)
             if w_descr is None:
-                selector = (name, DICT) # common case: no such attr in the class
+                attrname, index = (name, DICT) # common case: no such attr in the class
             elif isinstance(w_descr, MutableCell):
                 pass              # we have a MutableCell in the class: give up
             elif space.is_data_descr(w_descr):
@@ -888,20 +985,21 @@ def LOAD_ATTR_slowpath(pycode, w_obj, nameindex, map):
                 # (if any) has no relevance.
                 from pypy.interpreter.typedef import Member
                 if isinstance(w_descr, Member):    # it is a slot -- easy case
-                    selector = ("slot", SLOTS_STARTING_FROM + w_descr.index)
+                    attrname, index = ("slot", SLOTS_STARTING_FROM + w_descr.index)
             else:
                 # There is a non-data descriptor in the class.  If there is
                 # also a dict attribute, use the latter, caching its storageindex.
                 # If not, we loose.  We could do better in this case too,
                 # but we don't care too much; the common case of a method
                 # invocation is handled by LOOKUP_METHOD_xxx below.
-                selector = (name, DICT)
+                attrname = name
+                index = DICT
             #
-            if selector[1] != INVALID:
-                attr = map.find_map_attr(selector)
+            if index != INVALID:
+                attr = map.find_map_attr(attrname, index)
                 if attr is not None:
                     # Note that if map.terminator is a DevolvedDictTerminator,
-                    # map.find_map_attr will always return None if selector[1]==DICT.
+                    # map.find_map_attr will always return None if index==DICT.
                     _fill_cache(pycode, nameindex, map, version_tag, attr.storageindex)
                     return w_obj._mapdict_read_storage(attr.storageindex)
     if space.config.objspace.std.withmethodcachecounter:
