@@ -13,6 +13,7 @@ class ProcessorAutodetectError(Exception):
 MODEL_X86         = 'x86'
 MODEL_X86_NO_SSE2 = 'x86-without-sse2'
 MODEL_X86_64      = 'x86-64'
+MODEL_X86_64_SSE4 = 'x86-64-sse4'
 MODEL_ARM         = 'arm'
 MODEL_PPC_64      = 'ppc-64'
 # don't use '_' in the model strings; they are replaced by '-'
@@ -58,6 +59,8 @@ def detect_model_from_host_platform():
             'i86pc': MODEL_X86,    # Solaris/Intel
             'x86': MODEL_X86,      # Apple
             'Power Macintosh': MODEL_PPC_64,
+            'ppc64': MODEL_PPC_64,
+            'ppc64le': MODEL_PPC_64,
             'x86_64': MODEL_X86,
             'amd64': MODEL_X86,    # freebsd
             'AMD64': MODEL_X86,    # win64
@@ -70,18 +73,22 @@ def detect_model_from_host_platform():
         raise ProcessorAutodetectError, "unknown machine name %s" % mach
     #
     if result.startswith('x86'):
+        from rpython.jit.backend.x86 import detect_feature as feature
         if sys.maxint == 2**63-1:
             result = MODEL_X86_64
+            # has sse 2 at least
+            if feature.detect_sse4_1():
+                result = MODEL_X86_64_SSE4
         else:
             assert sys.maxint == 2**31-1
-            from rpython.jit.backend.x86 import detect_sse2
-            if detect_sse2.detect_sse2():
+            if feature.detect_sse2():
                 result = MODEL_X86
             else:
                 result = MODEL_X86_NO_SSE2
-            if detect_sse2.detect_x32_mode():
+            if feature.detect_x32_mode():
                 raise ProcessorAutodetectError(
                     'JITting in x32 mode is not implemented')
+
     #
     if result.startswith('arm'):
         from rpython.jit.backend.arm.detect import detect_float
@@ -109,8 +116,12 @@ def getcpuclassname(backend_name="auto"):
         return "rpython.jit.backend.x86.runner", "CPU386_NO_SSE2"
     elif backend_name == MODEL_X86_64:
         return "rpython.jit.backend.x86.runner", "CPU_X86_64"
+    elif backend_name == MODEL_X86_64_SSE4:
+        return "rpython.jit.backend.x86.runner", "CPU_X86_64_SSE4"
     elif backend_name == MODEL_ARM:
         return "rpython.jit.backend.arm.runner", "CPU_ARM"
+    elif backend_name == MODEL_PPC_64:
+        return "rpython.jit.backend.ppc.runner", "PPC_CPU"
     else:
         raise ProcessorAutodetectError, (
             "we have no JIT backend for this cpu: '%s'" % backend_name)
@@ -122,11 +133,16 @@ def getcpuclass(backend_name="auto"):
 
 
 def getcpufeatures(backend_name="auto"):
-    """NOT_RPYTHON"""
-    cpucls = getcpuclass(backend_name)
-    return [attr[len('supports_'):] for attr in dir(cpucls)
-                            if attr.startswith('supports_')
-                                and getattr(cpucls, attr)]
+    if backend_name == "auto":
+        backend_name = autodetect()
+    return {
+        MODEL_X86: ['floats', 'singlefloats', 'longlong'],
+        MODEL_X86_NO_SSE2: ['longlong'],
+        MODEL_X86_64: ['floats', 'singlefloats'],
+        MODEL_X86_64_SSE4: ['floats', 'singlefloats'],
+        MODEL_ARM: ['floats', 'singlefloats', 'longlong'],
+        MODEL_PPC_64: [], # we don't even have PPC directory, so no
+    }[backend_name]
 
 if __name__ == '__main__':
     if len(sys.argv) > 1:
