@@ -1,3 +1,4 @@
+import py
 from rpython.jit.backend.llsupport.descr import get_size_descr,\
      get_field_descr, get_array_descr, ArrayDescr, FieldDescr,\
      SizeDescr, get_interiorfield_descr
@@ -12,6 +13,8 @@ from rpython.jit.metainterp.history import AbstractFailDescr
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.rtyper import rclass
 from rpython.jit.backend.x86.arch import WORD
+from rpython.jit.backend.llsupport.symbolic import (WORD,
+        get_array_token)
 
 class Evaluator(object):
     def __init__(self, scope):
@@ -27,6 +30,27 @@ o_vtable = lltype.malloc(rclass.OBJECT_VTABLE, immortal=True)
 
 class RewriteTests(object):
     def check_rewrite(self, frm_operations, to_operations, **namespace):
+        def setfield(baseptr, newvalue, descr):
+            assert isinstance(baseptr, str)
+            assert isinstance(newvalue, (str, int))
+            assert not isinstance(descr, (str, int))
+            return 'gc_store(%s, %d, %s, %d)' % (baseptr, descr.offset,
+                                                 newvalue, descr.field_size)
+        def setarrayitem(baseptr, index, newvalue, descr):
+            assert isinstance(baseptr, str)
+            assert isinstance(index, (str, int))
+            assert isinstance(newvalue, (str, int))
+            assert not isinstance(descr, (str, int))
+            if isinstance(index, int):
+                offset = descr.basesize + index * descr.itemsize
+                return 'gc_store(%s, %d, %s, %d)' % (baseptr, offset,
+                                                     newvalue, descr.itemsize)
+            else:
+                return 'gc_store_indexed(%s, %s, %s, %d, %d, %s)' % (
+                    baseptr, index, newvalue,
+                    descr.itemsize, descr.basesize, descr.itemsize)
+        #
+        WORD = globals()['WORD']
         S = lltype.GcStruct('S', ('x', lltype.Signed),
                                  ('y', lltype.Signed))
         sdescr = get_size_descr(self.gc_ll_descr, S)
@@ -54,6 +78,26 @@ class RewriteTests(object):
         cdescr.tid = 8111
         clendescr = cdescr.lendescr
         #
+        S1 = lltype.GcStruct('S1')
+        S1I = lltype.GcArray(('x', lltype.Ptr(S1)),
+                             ('y', lltype.Ptr(S1)),
+                             ('z', lltype.Ptr(S1)))
+        itzdescr = get_interiorfield_descr(self.gc_ll_descr, S1I, 'z')
+        itydescr = get_interiorfield_descr(self.gc_ll_descr, S1I, 'y')
+        itxdescr = get_interiorfield_descr(self.gc_ll_descr, S1I, 'x')
+        S2I = lltype.GcArray(('x', lltype.Ptr(S1)),
+                             ('y', lltype.Ptr(S1)),
+                             ('z', lltype.Ptr(S1)),
+                             ('t', lltype.Ptr(S1)))   # size is a power of two
+        s2i_item_size_in_bits = (4 if WORD == 4 else 5)
+        ity2descr = get_interiorfield_descr(self.gc_ll_descr, S2I, 'y')
+        R1 = lltype.GcStruct('R', ('x', lltype.Signed),
+                                  ('y', lltype.Float),
+                                  ('z', lltype.Ptr(S1)))
+        xdescr = get_field_descr(self.gc_ll_descr, R1, 'x')
+        ydescr = get_field_descr(self.gc_ll_descr, R1, 'y')
+        zdescr = get_field_descr(self.gc_ll_descr, R1, 'z')
+        #
         E = lltype.GcStruct('Empty')
         edescr = get_size_descr(self.gc_ll_descr, E)
         edescr.tid = 9000
@@ -66,7 +110,13 @@ class RewriteTests(object):
         #
         tiddescr = self.gc_ll_descr.fielddescr_tid
         wbdescr = self.gc_ll_descr.write_barrier_descr
-        WORD = globals()['WORD']
+        #
+        F = lltype.GcArray(lltype.Float)
+        fdescr = get_array_descr(self.gc_ll_descr, F)
+        SF = lltype.GcArray(lltype.SingleFloat)
+        sfdescr = get_array_descr(self.gc_ll_descr, SF)
+        RAW_SF = lltype.Array(lltype.SingleFloat)
+        raw_sfdescr = get_array_descr(self.gc_ll_descr, RAW_SF)
         #
         strdescr     = self.gc_ll_descr.str_descr
         unicodedescr = self.gc_ll_descr.unicode_descr
@@ -124,6 +174,11 @@ class FakeTracker(object):
 class BaseFakeCPU(object):
     JITFRAME_FIXED_SIZE = 0
 
+    load_constant_offset = True
+    load_supported_factors = (1,2,4,8)
+
+    translate_support_code = None
+
     def __init__(self):
         self.tracker = FakeTracker()
         self._cache = {}
@@ -241,7 +296,7 @@ class TestBoehm(RewriteTests):
             [p1]
             p0 = call_malloc_gc(ConstClass(malloc_fixedsize), 102, \
                                 descr=malloc_fixedsize_descr)
-            setfield_gc(p0, ConstClass(o_vtable), descr=vtable_descr)
+            gc_store(p0, 0, ConstClass(o_vtable), %(vtable_descr.field_size)s)
             jump()
         """)
 
@@ -314,7 +369,7 @@ class TestFramework(RewriteTests):
         """, """
             [p1]
             p0 = call_malloc_nursery(%(sdescr.size)d)
-            setfield_gc(p0, 1234, descr=tiddescr)
+            gc_store(p0, 0, 1234, 8)
             jump()
         """)
 
@@ -329,12 +384,12 @@ class TestFramework(RewriteTests):
             []
             p0 = call_malloc_nursery(   \
                                %(sdescr.size + tdescr.size + sdescr.size)d)
-            setfield_gc(p0, 1234, descr=tiddescr)
+            gc_store(p0, 0, 1234, 8)
             p1 = nursery_ptr_increment(p0, %(sdescr.size)d)
-            setfield_gc(p1, 5678, descr=tiddescr)
+            gc_store(p1, 0, 5678, 8)
             p2 = nursery_ptr_increment(p1, %(tdescr.size)d)
-            setfield_gc(p2, 1234, descr=tiddescr)
-            zero_ptr_field(p1, %(tdescr.gc_fielddescrs[0].offset)s)
+            gc_store(p2, 0, 1234, 8)
+            %(setfield('p1', 0, tdescr.gc_fielddescrs[0]))s
             jump()
         """)
 
@@ -347,8 +402,8 @@ class TestFramework(RewriteTests):
             []
             p0 = call_malloc_nursery(    \
                                 %(adescr.basesize + 10 * adescr.itemsize)d)
-            setfield_gc(p0, 4321, descr=tiddescr)
-            setfield_gc(p0, 10, descr=alendescr)
+            gc_store(p0, 0, 4321, %(tiddescr.field_size)s)
+            gc_store(p0, 0, 10, %(alendescr.field_size)s)
             jump()
         """)
 
@@ -363,10 +418,10 @@ class TestFramework(RewriteTests):
             p0 = call_malloc_nursery(                                  \
                                 %(sdescr.size +                        \
                                   adescr.basesize + 10 * adescr.itemsize)d)
-            setfield_gc(p0, 1234, descr=tiddescr)
+            gc_store(p0, 0, 1234, %(tiddescr.field_size)s)
             p1 = nursery_ptr_increment(p0, %(sdescr.size)d)
-            setfield_gc(p1, 4321, descr=tiddescr)
-            setfield_gc(p1, 10, descr=alendescr)
+            gc_store(p1, 0, 4321, %(tiddescr.field_size)s)
+            gc_store(p1, 0, 10, %(alendescr.field_size)s)
             jump()
         """)
 
@@ -378,8 +433,8 @@ class TestFramework(RewriteTests):
         """, """
             []
             p0 = call_malloc_nursery(%(bdescr.basesize + 8)d)
-            setfield_gc(p0, 8765, descr=tiddescr)
-            setfield_gc(p0, 6, descr=blendescr)
+            gc_store(p0, 0, 8765, %(tiddescr.field_size)s)
+            gc_store(p0, 0, 6, %(blendescr.field_size)s)
             jump()
         """)
 
@@ -394,17 +449,17 @@ class TestFramework(RewriteTests):
         """, """
             []
             p0 = call_malloc_nursery(%(4 * (bdescr.basesize + 8))d)
-            setfield_gc(p0, 8765, descr=tiddescr)
-            setfield_gc(p0, 5, descr=blendescr)
+            gc_store(p0, 0, 8765, %(tiddescr.field_size)s)
+            gc_store(p0, 0, 5, %(blendescr.field_size)s)
             p1 = nursery_ptr_increment(p0, %(bdescr.basesize + 8)d)
-            setfield_gc(p1, 8765, descr=tiddescr)
-            setfield_gc(p1, 5, descr=blendescr)
+            gc_store(p1, 0, 8765, %(tiddescr.field_size)s)
+            gc_store(p1, 0, 5, %(blendescr.field_size)s)
             p2 = nursery_ptr_increment(p1, %(bdescr.basesize + 8)d)
-            setfield_gc(p2, 8765, descr=tiddescr)
-            setfield_gc(p2, 5, descr=blendescr)
+            gc_store(p2, 0, 8765, %(tiddescr.field_size)s)
+            gc_store(p2, 0, 5, %(blendescr.field_size)s)
             p3 = nursery_ptr_increment(p2, %(bdescr.basesize + 8)d)
-            setfield_gc(p3, 8765, descr=tiddescr)
-            setfield_gc(p3, 5, descr=blendescr)
+            gc_store(p3, 0, 8765, %(tiddescr.field_size)s)
+            gc_store(p3, 0, 5, %(blendescr.field_size)s)
             jump()
         """)
 
@@ -417,9 +472,9 @@ class TestFramework(RewriteTests):
         """, """
             []
             p0 = call_malloc_nursery(%(4*WORD)d)
-            setfield_gc(p0, 9000, descr=tiddescr)
+            gc_store(p0, 0,  9000, %(tiddescr.field_size)s)
             p1 = nursery_ptr_increment(p0, %(2*WORD)d)
-            setfield_gc(p1, 9000, descr=tiddescr)
+            gc_store(p1, 0,  9000, %(tiddescr.field_size)s)
             jump()
         """)
 
@@ -431,7 +486,7 @@ class TestFramework(RewriteTests):
         """, """
             [i0]
             p0 = call_malloc_nursery_varsize(0, 1, i0, descr=bdescr)
-            setfield_gc(p0, i0, descr=blendescr)
+            gc_store(p0, 0, i0, %(bdescr.basesize)s)
             jump(i0)
         """)
 
@@ -443,8 +498,8 @@ class TestFramework(RewriteTests):
         """, """
         [i0]
         p0 = call_malloc_nursery_varsize(1, 1, i0, descr=strdescr)
-        setfield_gc(p0, i0, descr=strlendescr)
-        setfield_gc(p0, 0, descr=strhashdescr)
+        gc_store(p0, %(strlendescr.offset)s, i0, %(strlendescr.field_size)s)
+        gc_store(p0, 0, 0, %(strlendescr.field_size)s)
         jump(i0)
         """)
 
@@ -457,10 +512,12 @@ class TestFramework(RewriteTests):
         nonstd_descr.basesize = 64      # <= hacked
         nonstd_descr.itemsize = 8
         nonstd_descr_gcref = 123
+        # REVIEW: added descr=nonstd_descr to setarrayitem
+        # is it even valid to have a setarrayitem WITHOUT a descr?
         self.check_rewrite("""
             [i0, p1]
             p0 = new_array(i0, descr=nonstd_descr)
-            setarrayitem_gc(p0, i0, p1)
+            setarrayitem_gc(p0, i0, p1, descr=nonstd_descr)
             jump(i0)
         """, """
             [i0, p1]
@@ -470,7 +527,7 @@ class TestFramework(RewriteTests):
                                 6464, i0,                             \
                                 descr=malloc_array_nonstandard_descr)
             cond_call_gc_wb_array(p0, i0, descr=wbdescr)
-            setarrayitem_gc(p0, i0, p1)
+            gc_store_indexed(p0, i0, p1, 8, 64, 8)
             jump(i0)
         """, nonstd_descr=nonstd_descr)
 
@@ -500,15 +557,15 @@ class TestFramework(RewriteTests):
             []
             p0 = call_malloc_nursery(    \
                               %(2 * (bdescr.basesize + 104))d)
-            setfield_gc(p0, 8765, descr=tiddescr)
-            setfield_gc(p0, 101, descr=blendescr)
+            gc_store(p0, 0,  8765, %(tiddescr.field_size)s)
+            gc_store(p0, 0,  101, %(blendescr.field_size)s)
             p1 = nursery_ptr_increment(p0, %(bdescr.basesize + 104)d)
-            setfield_gc(p1, 8765, descr=tiddescr)
-            setfield_gc(p1, 102, descr=blendescr)
+            gc_store(p1, 0,  8765, %(tiddescr.field_size)s)
+            gc_store(p1, 0,  102, %(blendescr.field_size)s)
             p2 = call_malloc_nursery(    \
                               %(bdescr.basesize + 104)d)
-            setfield_gc(p2, 8765, descr=tiddescr)
-            setfield_gc(p2, 103, descr=blendescr)
+            gc_store(p2, 0,  8765, %(tiddescr.field_size)s)
+            gc_store(p2, 0,  103, %(blendescr.field_size)s)
             jump()
         """)
 
@@ -534,8 +591,8 @@ class TestFramework(RewriteTests):
         """, """
             [p1]
             p0 = call_malloc_nursery(104)      # rounded up
-            setfield_gc(p0, 9315, descr=tiddescr)
-            setfield_gc(p0, 0, descr=vtable_descr)
+            gc_store(p0, 0,  9315, %(tiddescr.field_size)s)
+            gc_store(p0, 0,  0, %(vtable_descr.field_size)s)
             jump()
         """)
 
@@ -549,7 +606,7 @@ class TestFramework(RewriteTests):
             [p1]
             p0 = call_malloc_gc(ConstClass(malloc_big_fixedsize), 104, 9315, \
                                 descr=malloc_big_fixedsize_descr)
-            setfield_gc(p0, 0, descr=vtable_descr)
+            gc_store(p0, 0,  0, %(vtable_descr.field_size)s)
             jump()
         """)
 
@@ -566,21 +623,21 @@ class TestFramework(RewriteTests):
             p0 = call_malloc_nursery(                                \
                       %(strdescr.basesize + 16 * strdescr.itemsize + \
                         unicodedescr.basesize + 10 * unicodedescr.itemsize)d)
-            setfield_gc(p0, %(strdescr.tid)d, descr=tiddescr)
-            setfield_gc(p0, 14, descr=strlendescr)
-            setfield_gc(p0, 0, descr=strhashdescr)
+            gc_store(p0, 0,  %(strdescr.tid)d, %(tiddescr.field_size)s)
+            gc_store(p0, %(strlendescr.offset)s, 14, %(strlendescr.field_size)s)
+            gc_store(p0, 0,  0, %(strhashdescr.field_size)s)
             p1 = nursery_ptr_increment(p0, %(strdescr.basesize + 16 * strdescr.itemsize)d)
-            setfield_gc(p1, %(unicodedescr.tid)d, descr=tiddescr)
-            setfield_gc(p1, 10, descr=unicodelendescr)
-            setfield_gc(p1, 0, descr=unicodehashdescr)
+            gc_store(p1, 0,  %(unicodedescr.tid)d, %(tiddescr.field_size)s)
+            gc_store(p1, %(unicodelendescr.offset)s, 10, %(unicodelendescr.field_size)s)
+            gc_store(p1, 0,  0, %(unicodehashdescr.field_size)s)
             p2 = call_malloc_nursery_varsize(2, %(unicodedescr.itemsize)d, i2,\
                                 descr=unicodedescr)
-            setfield_gc(p2, i2, descr=unicodelendescr)
-            setfield_gc(p2, 0, descr=unicodehashdescr)
+            gc_store(p2, %(unicodelendescr.offset)s, i2, %(unicodelendescr.field_size)s)
+            gc_store(p2, 0,  0, %(unicodehashdescr.field_size)s)
             p3 = call_malloc_nursery_varsize(1, 1, i2, \
                                 descr=strdescr)
-            setfield_gc(p3, i2, descr=strlendescr)
-            setfield_gc(p3, 0, descr=strhashdescr)
+            gc_store(p3, %(strlendescr.offset)s, i2, %(strlendescr.field_size)s)
+            gc_store(p3, 0,  0, %(strhashdescr.field_size)s)
             jump()
         """)
 
@@ -592,7 +649,7 @@ class TestFramework(RewriteTests):
         """, """
             [p1, p2]
             cond_call_gc_wb(p1, descr=wbdescr)
-            setfield_gc(p1, p2, descr=tzdescr)
+            gc_store(p1, %(tzdescr.offset)s, p2, %(tzdescr.field_size)s)
             jump()
         """)
 
@@ -606,7 +663,7 @@ class TestFramework(RewriteTests):
         """, """
             [p1, i2, p3]
             cond_call_gc_wb(p1, descr=wbdescr)
-            setarrayitem_gc(p1, i2, p3, descr=cdescr)
+            %(setarrayitem('p1', 'i2', 'p3', cdescr))s
             jump()
         """)
 
@@ -622,12 +679,12 @@ class TestFramework(RewriteTests):
             [i2, p3]
             p1 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 129 * cdescr.itemsize)d)
-            setfield_gc(p1, 8111, descr=tiddescr)
-            setfield_gc(p1, 129, descr=clendescr)
+            gc_store(p1, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p1, 0,  129, %(clendescr.field_size)s)
             zero_array(p1, 0, 129, descr=cdescr)
             call_n(123456)
             cond_call_gc_wb(p1, descr=wbdescr)
-            setarrayitem_gc(p1, i2, p3, descr=cdescr)
+            %(setarrayitem('p1', 'i2', 'p3', cdescr))s
             jump()
         """)
 
@@ -644,12 +701,12 @@ class TestFramework(RewriteTests):
             [i2, p3]
             p1 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 130 * cdescr.itemsize)d)
-            setfield_gc(p1, 8111, descr=tiddescr)
-            setfield_gc(p1, 130, descr=clendescr)
+            gc_store(p1, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p1, 0,  130, %(clendescr.field_size)s)
             zero_array(p1, 0, 130, descr=cdescr)
             call_n(123456)
             cond_call_gc_wb_array(p1, i2, descr=wbdescr)
-            setarrayitem_gc(p1, i2, p3, descr=cdescr)
+            %(setarrayitem('p1', 'i2', 'p3', cdescr))s
             jump()
         """)
 
@@ -661,7 +718,7 @@ class TestFramework(RewriteTests):
         """, """
             [p1, i2, p3]
             cond_call_gc_wb_array(p1, i2, descr=wbdescr)
-            setarrayitem_gc(p1, i2, p3, descr=cdescr)
+            %(setarrayitem('p1', 'i2', 'p3', cdescr))s
             jump()
         """)
 
@@ -676,12 +733,12 @@ class TestFramework(RewriteTests):
             [i2, p3]
             p1 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 5 * cdescr.itemsize)d)
-            setfield_gc(p1, 8111, descr=tiddescr)
-            setfield_gc(p1, 5, descr=clendescr)
+            gc_store(p1, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p1, 0,  5, %(clendescr.field_size)s)
             zero_array(p1, 0, 5, descr=cdescr)
             label(p1, i2, p3)
             cond_call_gc_wb_array(p1, i2, descr=wbdescr)
-            setarrayitem_gc(p1, i2, p3, descr=cdescr)
+            %(setarrayitem('p1', 'i2', 'p3', cdescr))s
             jump()
         """)
 
@@ -693,16 +750,21 @@ class TestFramework(RewriteTests):
         interiorlendescr = interiordescr.lendescr
         interiorzdescr = get_interiorfield_descr(self.gc_ll_descr,
                                                  INTERIOR, 'z')
+        scale = interiorzdescr.arraydescr.itemsize
+        offset = interiorzdescr.arraydescr.basesize
+        offset += interiorzdescr.fielddescr.offset
+        size = interiorzdescr.arraydescr.itemsize
         self.check_rewrite("""
             [p1, p2]
-            setinteriorfield_gc(p1, 0, p2, descr=interiorzdescr)
+            setinteriorfield_gc(p1, 7, p2, descr=interiorzdescr)
             jump(p1, p2)
         """, """
             [p1, p2]
-            cond_call_gc_wb_array(p1, 0, descr=wbdescr)
-            setinteriorfield_gc(p1, 0, p2, descr=interiorzdescr)
+            cond_call_gc_wb_array(p1, 7, descr=wbdescr)
+            gc_store(p1, %(offset + 7 * scale)s, p2, %(size)s)
             jump(p1, p2)
-        """, interiorzdescr=interiorzdescr)
+        """, interiorzdescr=interiorzdescr, scale=scale,
+             offset=offset, size=size)
 
     def test_initialization_store(self):
         self.check_rewrite("""
@@ -713,8 +775,8 @@ class TestFramework(RewriteTests):
         """, """
             [p1]
             p0 = call_malloc_nursery(%(tdescr.size)d)
-            setfield_gc(p0, 5678, descr=tiddescr)
-            setfield_gc(p0, p1, descr=tzdescr)
+            gc_store(p0, 0,  5678, %(tiddescr.field_size)s)
+            gc_store(p0, %(tzdescr.offset)s, p1, %(tzdescr.field_size)s)
             jump()
         """)
 
@@ -728,11 +790,11 @@ class TestFramework(RewriteTests):
         """, """
             []
             p0 = call_malloc_nursery(%(tdescr.size + sdescr.size)d)
-            setfield_gc(p0, 5678, descr=tiddescr)
+            gc_store(p0, 0,  5678, %(tiddescr.field_size)s)
             p1 = nursery_ptr_increment(p0, %(tdescr.size)d)
-            setfield_gc(p1, 1234, descr=tiddescr)
+            gc_store(p1, 0,  1234, %(tiddescr.field_size)s)
             # <<<no cond_call_gc_wb here>>>
-            setfield_gc(p0, p1, descr=tzdescr)
+            gc_store(p0, %(tzdescr.offset)s, p1, %(tzdescr.field_size)s)
             jump()
         """)
 
@@ -746,10 +808,10 @@ class TestFramework(RewriteTests):
             [p1, i2]
             p0 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 5 * cdescr.itemsize)d)
-            setfield_gc(p0, 8111, descr=tiddescr)
-            setfield_gc(p0, 5, descr=clendescr)
+            gc_store(p0, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p0, 0,  5, %(clendescr.field_size)s)
             zero_array(p0, 0, 5, descr=cdescr)
-            setarrayitem_gc(p0, i2, p1, descr=cdescr)
+            %(setarrayitem('p0', 'i2', 'p1', cdescr))s
             jump()
         """)
 
@@ -764,11 +826,11 @@ class TestFramework(RewriteTests):
             [p1, p2]
             p0 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 5 * cdescr.itemsize)d)
-            setfield_gc(p0, 8111, descr=tiddescr)
-            setfield_gc(p0, 5, descr=clendescr)
+            gc_store(p0, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p0, 0,  5, %(clendescr.field_size)s)
             zero_array(p0, 2, 3, descr=cdescr)
-            setarrayitem_gc(p0, 1, p1, descr=cdescr)
-            setarrayitem_gc(p0, 0, p2, descr=cdescr)
+            %(setarrayitem('p0', 1, 'p1', cdescr))s
+            %(setarrayitem('p0', 0, 'p2', cdescr))s
             jump()
         """)
 
@@ -783,11 +845,11 @@ class TestFramework(RewriteTests):
             [p1, p2]
             p0 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 5 * cdescr.itemsize)d)
-            setfield_gc(p0, 8111, descr=tiddescr)
-            setfield_gc(p0, 5, descr=clendescr)
+            gc_store(p0, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p0, 0,  5, %(clendescr.field_size)s)
             zero_array(p0, 0, 3, descr=cdescr)
-            setarrayitem_gc(p0, 3, p1, descr=cdescr)
-            setarrayitem_gc(p0, 4, p2, descr=cdescr)
+            %(setarrayitem('p0', 3, 'p1', cdescr))s
+            %(setarrayitem('p0', 4, 'p2', cdescr))s
             jump()
         """)
 
@@ -803,12 +865,12 @@ class TestFramework(RewriteTests):
             [p1, p2]
             p0 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 5 * cdescr.itemsize)d)
-            setfield_gc(p0, 8111, descr=tiddescr)
-            setfield_gc(p0, 5, descr=clendescr)
+            gc_store(p0, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p0, 0,  5, %(clendescr.field_size)s)
             zero_array(p0, 0, 5, descr=cdescr)
-            setarrayitem_gc(p0, 3, p1, descr=cdescr)
-            setarrayitem_gc(p0, 2, p2, descr=cdescr)
-            setarrayitem_gc(p0, 1, p2, descr=cdescr)
+            %(setarrayitem('p0', 3, 'p1', cdescr))s
+            %(setarrayitem('p0', 2, 'p2', cdescr))s
+            %(setarrayitem('p0', 1, 'p2', cdescr))s
             jump()
         """)
 
@@ -826,14 +888,14 @@ class TestFramework(RewriteTests):
             [p1, p2]
             p0 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 5 * cdescr.itemsize)d)
-            setfield_gc(p0, 8111, descr=tiddescr)
-            setfield_gc(p0, 5, descr=clendescr)
+            gc_store(p0, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p0, 0,  5, %(clendescr.field_size)s)
             zero_array(p0, 5, 0, descr=cdescr)
-            setarrayitem_gc(p0, 3, p1, descr=cdescr)
-            setarrayitem_gc(p0, 4, p2, descr=cdescr)
-            setarrayitem_gc(p0, 0, p1, descr=cdescr)
-            setarrayitem_gc(p0, 2, p2, descr=cdescr)
-            setarrayitem_gc(p0, 1, p2, descr=cdescr)
+            %(setarrayitem('p0', 3, 'p1', cdescr))s
+            %(setarrayitem('p0', 4, 'p2', cdescr))s
+            %(setarrayitem('p0', 0, 'p1', cdescr))s
+            %(setarrayitem('p0', 2, 'p2', cdescr))s
+            %(setarrayitem('p0', 1, 'p2', cdescr))s
             jump()
         """)
 
@@ -849,13 +911,13 @@ class TestFramework(RewriteTests):
             [p1, p2]
             p0 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 5 * cdescr.itemsize)d)
-            setfield_gc(p0, 8111, descr=tiddescr)
-            setfield_gc(p0, 5, descr=clendescr)
+            gc_store(p0, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p0, 0,  5, %(clendescr.field_size)s)
             zero_array(p0, 1, 4, descr=cdescr)
-            setarrayitem_gc(p0, 0, p1, descr=cdescr)
+            %(setarrayitem('p0', 0, 'p1', cdescr))s
             call_n(321321)
             cond_call_gc_wb(p0, descr=wbdescr)
-            setarrayitem_gc(p0, 1, p2, descr=cdescr)
+            %(setarrayitem('p0', 1, 'p2', cdescr))s
             jump()
         """)
 
@@ -871,13 +933,13 @@ class TestFramework(RewriteTests):
             [p1, p2]
             p0 = call_malloc_nursery(    \
                                 %(cdescr.basesize + 5 * cdescr.itemsize)d)
-            setfield_gc(p0, 8111, descr=tiddescr)
-            setfield_gc(p0, 5, descr=clendescr)
+            gc_store(p0, 0,  8111, %(tiddescr.field_size)s)
+            gc_store(p0, 0,  5, %(clendescr.field_size)s)
             zero_array(p0, 1, 4, descr=cdescr)
-            setarrayitem_gc(p0, 0, p1, descr=cdescr)
+            %(setarrayitem('p0', 0, 'p1', cdescr))s
             label(p0, p2)
             cond_call_gc_wb_array(p0, 1, descr=wbdescr)
-            setarrayitem_gc(p0, 1, p2, descr=cdescr)
+            %(setarrayitem('p0', 1, 'p2', cdescr))s
             jump()
         """)
 
@@ -889,7 +951,7 @@ class TestFramework(RewriteTests):
         """, """
             [p1, p2, i3]
             p0 = call_malloc_nursery_varsize(0, 1, i3, descr=bdescr)
-            setfield_gc(p0, i3, descr=blendescr)
+            gc_store(p0, 0,  i3, %(blendescr.field_size)s)
             zero_array(p0, 0, i3, descr=bdescr)
             jump()
         """)
@@ -903,10 +965,10 @@ class TestFramework(RewriteTests):
         """, """
             [p1, p2, i3]
             p0 = call_malloc_nursery_varsize(0, 1, i3, descr=bdescr)
-            setfield_gc(p0, i3, descr=blendescr)
+            gc_store(p0, 0,  i3, %(blendescr.field_size)s)
             zero_array(p0, 0, i3, descr=bdescr)
             cond_call_gc_wb_array(p0, 0, descr=wbdescr)
-            setarrayitem_gc(p0, 0, p1, descr=bdescr)
+            %(setarrayitem('p0', 0, 'p1', bdescr))s
             jump()
         """)
 
@@ -922,9 +984,9 @@ class TestFramework(RewriteTests):
         """, """
             [i0, p1, i2]
             p0 = call_malloc_nursery_varsize(0, 1, i0, descr=bdescr)
-            setfield_gc(p0, i0, descr=blendescr)
+            gc_store(p0, 0,  i0, %(blendescr.field_size)s)
             cond_call_gc_wb_array(p0, i2, descr=wbdescr)
-            setarrayitem_gc(p0, i2, p1, descr=bdescr)
+            gc_store_indexed(p0, i2, p1, 1, %(bdescr.basesize)s, 1)
             jump()
         """)
 
@@ -938,14 +1000,14 @@ class TestFramework(RewriteTests):
         """, """
             [i0]
             p0 = call_malloc_nursery(%(tdescr.size)d)
-            setfield_gc(p0, 5678, descr=tiddescr)
-            zero_ptr_field(p0, %(tdescr.gc_fielddescrs[0].offset)s)
+            gc_store(p0, 0,  5678, %(tiddescr.field_size)s)
+            gc_store(p0, %(tdescr.gc_fielddescrs[0].offset)s, 0, %(tdescr.gc_fielddescrs[0].offset)s)
             p1 = call_malloc_nursery_varsize(1, 1, i0, \
                                 descr=strdescr)
-            setfield_gc(p1, i0, descr=strlendescr)
-            setfield_gc(p1, 0, descr=strhashdescr)
+            gc_store(p1, %(strlendescr.offset)s, i0, %(strlendescr.field_size)s)
+            gc_store(p1, 0,  0, %(strhashdescr.field_size)s)
             cond_call_gc_wb(p0, descr=wbdescr)
-            setfield_gc(p0, p1, descr=tzdescr)
+            gc_store(p0, %(tzdescr.offset)s, p1, %(tzdescr.field_size)s)
             jump()
         """)
 
@@ -959,11 +1021,11 @@ class TestFramework(RewriteTests):
         """, """
             [p1]
             p0 = call_malloc_nursery(%(tdescr.size)d)
-            setfield_gc(p0, 5678, descr=tiddescr)
-            zero_ptr_field(p0, %(tdescr.gc_fielddescrs[0].offset)s)
+            gc_store(p0, 0,  5678, %(tiddescr.field_size)s)
+            gc_store(p0, %(tdescr.gc_fielddescrs[0].offset)s, 0, %(tdescr.gc_fielddescrs[0].offset)s)
             label(p0, p1)
             cond_call_gc_wb(p0, descr=wbdescr)
-            setfield_gc(p0, p1, descr=tzdescr)
+            gc_store(p0, %(tzdescr.offset)s, p1, %(tzdescr.field_size)s)
             jump()
         """)
 
@@ -976,8 +1038,8 @@ class TestFramework(RewriteTests):
         """, """
             [p0, p1, p2]
             cond_call_gc_wb(p0, descr=wbdescr)
-            setfield_gc(p0, p1, descr=tzdescr)
-            setfield_gc(p0, p2, descr=tzdescr)
+            gc_store(p0, %(tzdescr.offset)s, p1, %(tzdescr.field_size)s)
+            gc_store(p0, %(tzdescr.offset)s, p2, %(tzdescr.field_size)s)
             jump(p1, p2, p0)
         """)
 
@@ -987,20 +1049,20 @@ class TestFramework(RewriteTests):
         i2 = call_assembler_i(i0, f0, descr=casmdescr)
         """, """
         [i0, f0]
-        i1 = getfield_raw_i(ConstClass(frame_info), descr=jfi_frame_size)
+        i1 = gc_load_i(ConstClass(frame_info), %(jfi_frame_size.offset)s, %(jfi_frame_size.field_size)s)
         p1 = call_malloc_nursery_varsize_frame(i1)
-        setfield_gc(p1, 0, descr=tiddescr)
-        i2 = getfield_raw_i(ConstClass(frame_info), descr=jfi_frame_depth)
-        setfield_gc(p1, 0, descr=jf_extra_stack_depth)
-        setfield_gc(p1, NULL, descr=jf_savedata)
-        setfield_gc(p1, NULL, descr=jf_force_descr)
-        setfield_gc(p1, NULL, descr=jf_descr)
-        setfield_gc(p1, NULL, descr=jf_guard_exc)
-        setfield_gc(p1, NULL, descr=jf_forward)
-        setfield_gc(p1, i2, descr=framelendescr)
-        setfield_gc(p1, ConstClass(frame_info), descr=jf_frame_info)
-        setarrayitem_gc(p1, 0, i0, descr=signedframedescr)
-        setarrayitem_gc(p1, 1, f0, descr=floatframedescr)
+        gc_store(p1, 0,  0, %(tiddescr.field_size)s)
+        i2 = gc_load_i(ConstClass(frame_info), %(jfi_frame_depth.offset)s, %(jfi_frame_depth.field_size)s)
+        %(setfield('p1', 0, jf_extra_stack_depth))s
+        %(setfield('p1', 'NULL', jf_savedata))s
+        %(setfield('p1', 'NULL', jf_force_descr))s
+        %(setfield('p1', 'NULL', jf_descr))s
+        %(setfield('p1', 'NULL', jf_guard_exc))s
+        %(setfield('p1', 'NULL', jf_forward))s
+        gc_store(p1, 0, i2, %(framelendescr.field_size)s)
+        %(setfield('p1', 'ConstClass(frame_info)', jf_frame_info))s
+        gc_store(p1, 3, i0, 8)
+        gc_store(p1, 13, f0, 8)
         i3 = call_assembler_i(p1, descr=casmdescr)
         """)
 
@@ -1014,8 +1076,8 @@ class TestFramework(RewriteTests):
         """, """
             [i0]
             p0 = call_malloc_nursery(%(tdescr.size)d)
-            setfield_gc(p0, 5678, descr=tiddescr)
-            zero_ptr_field(p0, %(tdescr.gc_fielddescrs[0].offset)s)
+            gc_store(p0, 0,  5678, %(tiddescr.field_size)s)
+            gc_store(p0, %(tdescr.gc_fielddescrs[0].offset)s, 0, %(tdescr.gc_fielddescrs[0].offset)s)
             i1 = int_add_ovf(i0, 123)
             guard_overflow(descr=guarddescr) []
             jump()
@@ -1031,8 +1093,8 @@ class TestFramework(RewriteTests):
         """, """
             [i0]
             p0 = call_malloc_nursery(%(tdescr.size)d)
-            setfield_gc(p0, 5678, descr=tiddescr)
-            zero_ptr_field(p0, %(tdescr.gc_fielddescrs[0].offset)s)
+            gc_store(p0, 0,  5678, %(tiddescr.field_size)s)
+            gc_store(p0, %(tdescr.gc_fielddescrs[0].offset)s, 0, %(tdescr.gc_fielddescrs[0].offset)s)
             i1 = int_gt(i0, 123)
             guard_false(i1, descr=guarddescr) []
             jump()
@@ -1045,14 +1107,14 @@ class TestFramework(RewriteTests):
         self.check_rewrite("""
             []
             p0 = new(descr=tdescr)
-            p1 = getfield_gc_r(p0, descr=tdescr)
+            p1 = getfield_gc_r(p0, descr=tzdescr)
             jump(p1)
         """, """
             []
             p0 = call_malloc_nursery(%(tdescr.size)d)
-            setfield_gc(p0, 5678, descr=tiddescr)
-            zero_ptr_field(p0, %(tdescr.gc_fielddescrs[0].offset)s)
-            p1 = getfield_gc_r(p0, descr=tdescr)
+            gc_store(p0, 0,  5678, %(tiddescr.field_size)s)
+            gc_store(p0, %(tdescr.gc_fielddescrs[0].offset)s, 0, %(tdescr.gc_fielddescrs[0].offset)s)
+            p1 = gc_load_r(p0, %(tzdescr.offset)s, %(tzdescr.field_size)s)
             jump(p1)
         """)
 
@@ -1081,3 +1143,126 @@ class TestFramework(RewriteTests):
             guard_false(i2) [i5, i0]
             jump()
         """)
+
+    @py.test.mark.parametrize('support_offset,factors,fromto',[
+        # [False, (1,2,4,8), 'setarrayitem_gc(p0,i1,i2,descr=adescr)' '->'
+        #    'i3 = int_mul(i1,%(adescr.itemsize)s);'
+        #    'i4 = int_add(i3,%(adescr.basesize)s);'
+        #    'gc_store(p0,i4,i2,%(adescr.itemsize)s)'],
+        [True, (1,2,4,8), 'setarrayitem_gc(p0,i1,i2,descr=adescr)' '->'
+           'gc_store_indexed(p0,i1,i2,%(adescr.itemsize)s,'
+           '%(adescr.basesize)s,%(adescr.itemsize)s)'],
+        #[False, (1,), 'setarrayitem_gc(p0,i1,i2,descr=adescr)' '->'
+        #   'i3 = int_mul(i1,%(adescr.itemsize)s);'
+        #   'i4 = int_add(i3,%(adescr.basesize)s);'
+        #   'gc_store(p0,i4,i2,%(adescr.itemsize)s)'],
+        [True, None, 'i3 = raw_load_i(p0,i1,descr=adescr)' '->'
+           'gc_load_indexed_i(p0,i1,1,%(adescr.basesize)s,-%(adescr.itemsize)s)'],
+        [True, None, 'i3 = raw_load_f(p0,i1,descr=fdescr)' '->'
+           'gc_load_indexed_f(p0,i1,1,%(fdescr.basesize)s,%(fdescr.itemsize)s)'],
+        [True, None, 'i3 = raw_load_i(p0,i1,descr=sfdescr)' '->'
+           'gc_load_indexed_i(p0,i1,1,%(sfdescr.basesize)s,%(sfdescr.itemsize)s)'],
+        [True, (1,2,4,8), 'i3 = raw_store(p0,i1,i2,descr=raw_sfdescr)' '->'
+           'gc_store_indexed(p0,i1,i2,1,%(raw_sfdescr.basesize)s,%(raw_sfdescr.itemsize)s)'],
+        # [False, (1,), 'i3 = raw_store(p0,i1,i2,descr=raw_sfdescr)' '->'
+        #    'i5 = int_add(i1,%(raw_sfdescr.basesize)s);'
+        #    'gc_store(p0,i5,i2,%(raw_sfdescr.itemsize)s)'],
+        [True, (1,2,4,8), 'i3 = getfield_gc_f(p0,descr=ydescr)' '->'
+           'i3 = gc_load_f(p0,%(ydescr.offset)s,%(ydescr.field_size)s)'],
+        [True, (1,2,4,8), 'setfield_raw(p0,i1,descr=ydescr)' '->'
+           'gc_store(p0,%(ydescr.offset)s,i1,%(ydescr.field_size)s)'],
+        [True, (1,2,4,8), 'setfield_gc(p0,p0,descr=zdescr)' '->'
+           'cond_call_gc_wb(p0, descr=wbdescr);'
+           'gc_store(p0,%(zdescr.offset)s,p0,%(zdescr.field_size)s)'],
+        [False, (1,), 'i3 = arraylen_gc(p0, descr=adescr)' '->'
+                      'i3 = gc_load_i(p0,0,%(adescr.itemsize)s)'],
+        #[False, (1,),  'i3 = strlen(p0)' '->'
+        #               'i3 = gc_load_i(p0,'
+        #               '%(strlendescr.offset)s,%(strlendescr.field_size)s)'],
+        [True,  (1,),  'i3 = strlen(p0)' '->'
+                       'i3 = gc_load_i(p0,'
+                                 '%(strlendescr.offset)s,'
+                                 '%(strlendescr.field_size)s)'],
+        #[False, (1,),  'i3 = unicodelen(p0)' '->'
+        #               'i3 = gc_load_i(p0,'
+        #                       '%(unicodelendescr.offset)s,'
+        #                       '%(unicodelendescr.field_size)s)'],
+        [True,  (1,),  'i3 = unicodelen(p0)' '->'
+                       'i3 = gc_load_i(p0,'
+                               '%(unicodelendescr.offset)s,'
+                               '%(unicodelendescr.field_size)s)'],
+
+        ## getitem str/unicode
+        [True,  (4,),  'i3 = unicodegetitem(p0,i1)' '->'
+                       'i3 = gc_load_indexed_i(p0,i1,'
+                                  '%(unicodedescr.itemsize)d,'
+                                  '%(unicodedescr.basesize)d,'
+                                  '%(unicodedescr.itemsize)d)'],
+        #[False, (4,),  'i3 = unicodegetitem(p0,i1)' '->'
+        #               'i4 = int_mul(i1, %(unicodedescr.itemsize)d);'
+        #               'i5 = int_add(i4, %(unicodedescr.basesize)d);'
+        #               'i3 = gc_load_i(p0,i5,%(unicodedescr.itemsize)d)'],
+        [True,  (4,),  'i3 = strgetitem(p0,i1)' '->'
+                       'i3 = gc_load_indexed_i(p0,i1,1,'
+                       '%(strdescr.basesize)d,1)'],
+        #[False, (4,),  'i3 = strgetitem(p0,i1)' '->'
+        #               'i5 = int_add(i1, %(strdescr.basesize)d);'
+        #               'i3 = gc_load_i(p0,i5,1)'],
+        ## setitem str/unicode
+        [True, (4,),  'i3 = strsetitem(p0,i1,0)' '->'
+                      'i3 = gc_store_indexed(p0,i1,0,1,'
+                               '%(strdescr.basesize)d,1)'],
+        [True, (4,),  'i3 = unicodesetitem(p0,i1,0)' '->'
+                      'i3 = gc_store_indexed(p0,i1,0,'
+                                 '%(unicodedescr.itemsize)d,'
+                                 '%(unicodedescr.basesize)d,'
+                                 '%(unicodedescr.itemsize)d)'],
+        ## interior
+        [True, (1,2,4,8), 'i3 = getinteriorfield_gc_i(p0,i1,descr=itzdescr)' '->'
+                          'i4 = int_mul(i1,'
+                             '%(itzdescr.arraydescr.itemsize)d);'
+                          'i3 = gc_load_indexed_i(p0,i4,1,'
+                                   '%(itzdescr.arraydescr.basesize'
+                                   '   + itzdescr.fielddescr.offset)d,'
+                                   '%(itzdescr.fielddescr.field_size)d)'],
+        [True, (1,2,4,8), 'i3 = getinteriorfield_gc_r(p0,i1,descr=itxdescr)' '->'
+                          'i4 = int_mul(i1,'
+                             '%(itxdescr.arraydescr.itemsize)d);'
+                          'i3 = gc_load_indexed_r(p0,i4,1,'
+                             '%(itxdescr.arraydescr.basesize'
+                             '   + itxdescr.fielddescr.offset)d,'
+                             '%(itxdescr.fielddescr.field_size)d)'],
+        [True, (1,2,4,8), 'i3 = setinteriorfield_gc(p0,i1,i2,descr=itydescr)' '->'
+                          'i4 = int_mul(i1,'
+                             '%(itydescr.arraydescr.itemsize)d);'
+                          'i3 = gc_store_indexed(p0,i4,i2,1,'
+                             '%(itydescr.arraydescr.basesize'
+                             '   + itydescr.fielddescr.offset)d,'
+                             '%(itydescr.fielddescr.field_size)d)'],
+        [True, (1,2,4,8), 'i3 = setinteriorfield_gc(p0,i1,i2,descr=ity2descr)' '->'
+                          'i4 = int_lshift(i1,'
+                             '%(s2i_item_size_in_bits)d);'
+                          'i3 = gc_store_indexed(p0,i4,i2,1,'
+                             '%(ity2descr.arraydescr.basesize'
+                             '   + itydescr.fielddescr.offset)d,'
+                             '%(ity2descr.fielddescr.field_size)d)'],
+    ])
+    def test_gc_load_store_transform(self, support_offset, factors, fromto):
+        self.cpu.load_constant_offset = support_offset
+        all_supported_sizes = [factors]
+
+        if not factors:
+            all_supported_sizes = [(1,), (1,2,), (4,), (1,2,4,8)]
+        for factors in all_supported_sizes:
+            self.cpu.load_supported_factors = factors
+            f, t = fromto.split('->')
+            t = ('\n' +(' '*16)).join([s for s in t.split(';')])
+            self.check_rewrite("""
+                [p0,i1,i2]
+                {f}
+                jump()
+            """.format(**locals()), """
+                [p0,i1,i2]
+                {t}
+                jump()
+            """.format(**locals()))
