@@ -4,8 +4,7 @@
 
 import os, sys
 from rpython.jit.backend.llsupport import symbolic
-from rpython.jit.backend.llsupport.descr import (ArrayDescr, CallDescr,
-    unpack_arraydescr, unpack_fielddescr, unpack_interiorfielddescr)
+from rpython.jit.backend.llsupport.descr import CallDescr, unpack_arraydescr
 from rpython.jit.backend.llsupport.gcmap import allocate_gcmap
 from rpython.jit.backend.llsupport.regalloc import (FrameManager, BaseRegalloc,
      RegisterManager, TempVar, compute_vars_longevity, is_comparison_or_ovf_op,
@@ -372,6 +371,8 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
             i += 1
         assert not self.rm.reg_bindings
         assert not self.xrm.reg_bindings
+        if not we_are_translated():
+            self.assembler.mc.UD2()
         self.flush_loop()
         self.assembler.mc.mark_op(None) # end of the loop
         self.operations = None
@@ -408,21 +409,15 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
             self.assembler.guard_success_cc = rx86.Conditions['NZ']
 
 
-    def _consider_guard_cc(true):
-        def consider_guard_cc(self, op):
-            arg = op.getarg(0)
-            if arg.is_vector():
-                loc = self.loc(arg)
-                self.assembler.guard_vector(op, self.loc(arg), true)
-            else:
-                self.load_condition_into_cc(arg)
-            self.perform_guard(op, [], None)
-        return consider_guard_cc
+    def _consider_guard_cc(self, op):
+        arg = op.getarg(0)
+        self.load_condition_into_cc(arg)
+        self.perform_guard(op, [], None)
 
-    consider_guard_true = _consider_guard_cc(True)
-    consider_guard_false = _consider_guard_cc(False)
-    consider_guard_nonnull = _consider_guard_cc(True)
-    consider_guard_isnull = _consider_guard_cc(False)
+    consider_guard_true = _consider_guard_cc
+    consider_guard_false = _consider_guard_cc
+    consider_guard_nonnull = _consider_guard_cc
+    consider_guard_isnull = _consider_guard_cc
 
     def consider_finish(self, op):
         # the frame is in ebp, but we have to point where in the frame is
@@ -921,8 +916,8 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
         args = op.getarglist()
         N = len(args)
         # we force all arguments in a reg (unless they are Consts),
-        # because it will be needed anyway by the following setfield_gc
-        # or setarrayitem_gc. It avoids loading it twice from the memory.
+        # because it will be needed anyway by the following gc_load
+        # It avoids loading it twice from the memory.
         arglocs = [self.rm.make_sure_var_in_reg(op.getarg(i), args)
                    for i in range(N)]
         self.perform_discard(op, arglocs)
@@ -1038,78 +1033,14 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
                 gcmap[val // WORD // 8] |= r_uint(1) << (val % (WORD * 8))
         return gcmap
 
-    def consider_setfield_gc(self, op):
-        ofs, size, _ = unpack_fielddescr(op.getdescr())
-        ofs_loc = imm(ofs)
-        size_loc = imm(size)
-        assert isinstance(size_loc, ImmedLoc)
-        if size_loc.value == 1:
-            need_lower_byte = True
-        else:
-            need_lower_byte = False
+    def consider_gc_store(self, op):
         args = op.getarglist()
         base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), args)
-        value_loc = self.make_sure_var_in_reg(op.getarg(1), args,
-                                              need_lower_byte=need_lower_byte)
-        self.perform_discard(op, [base_loc, ofs_loc, size_loc, value_loc])
-
-    def consider_zero_ptr_field(self, op):
-        ofs_loc = imm(op.getarg(1).getint())
-        size_loc = imm(WORD)
-        base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), [])
-        value_loc = imm(0)
-        self.perform_discard(op, [base_loc, ofs_loc, size_loc, value_loc])
-
-    consider_setfield_raw = consider_setfield_gc
-
-    def consider_setinteriorfield_gc(self, op):
-        t = unpack_interiorfielddescr(op.getdescr())
-        ofs, itemsize, fieldsize = imm(t[0]), imm(t[1]), imm(t[2])
-        args = op.getarglist()
-        if fieldsize.value == 1:
-            need_lower_byte = True
-        else:
-            need_lower_byte = False
-        box_base, box_index, box_value = args
-        base_loc = self.rm.make_sure_var_in_reg(box_base, args)
-        index_loc = self.rm.make_sure_var_in_reg(box_index, args)
-        value_loc = self.make_sure_var_in_reg(box_value, args,
-                                              need_lower_byte=need_lower_byte)
-        # If 'index_loc' is not an immediate, then we need a 'temp_loc' that
-        # is a register whose value will be destroyed.  It's fine to destroy
-        # the same register as 'index_loc', but not the other ones.
-        if not isinstance(index_loc, ImmedLoc):
-            # ...that is, except in a corner case where 'index_loc' would be
-            # in the same register as 'value_loc'...
-            tempvar = TempVar()
-            temp_loc = self.rm.force_allocate_reg(tempvar, [box_base,
-                                                            box_value])
-            self.rm.possibly_free_var(tempvar)
-        else:
-            temp_loc = None
-        self.rm.possibly_free_var(box_index)
-        self.rm.possibly_free_var(box_base)
-        self.possibly_free_var(box_value)
-        self.perform_discard(op, [base_loc, ofs, itemsize, fieldsize,
-                                 index_loc, temp_loc, value_loc])
-
-    consider_setinteriorfield_raw = consider_setinteriorfield_gc
-
-    def consider_strsetitem(self, op):
-        args = op.getarglist()
-        base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), args)
-        ofs_loc = self.rm.make_sure_var_in_reg(op.getarg(1), args)
-        value_loc = self.rm.make_sure_var_in_reg(op.getarg(2), args,
-                                                 need_lower_byte=True)
-        self.perform_discard(op, [base_loc, ofs_loc, value_loc])
-
-    consider_unicodesetitem = consider_strsetitem
-
-    def consider_setarrayitem_gc(self, op):
-        itemsize, ofs, _ = unpack_arraydescr(op.getdescr())
-        args = op.getarglist()
-        base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), args)
-        if itemsize == 1:
+        size_box = op.getarg(3)
+        assert isinstance(size_box, ConstInt)
+        size = size_box.value
+        assert size >= 1
+        if size == 1:
             need_lower_byte = True
         else:
             need_lower_byte = False
@@ -1117,92 +1048,79 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
                                           need_lower_byte=need_lower_byte)
         ofs_loc = self.rm.make_sure_var_in_reg(op.getarg(1), args)
         self.perform_discard(op, [base_loc, ofs_loc, value_loc,
-                                 imm(itemsize), imm(ofs)])
+                                 imm(size)])
 
-    consider_setarrayitem_raw = consider_setarrayitem_gc
-    consider_raw_store = consider_setarrayitem_gc
-
-    def _consider_getfield(self, op):
-        ofs, size, sign = unpack_fielddescr(op.getdescr())
-        ofs_loc = imm(ofs)
-        size_loc = imm(size)
+    def consider_gc_store_indexed(self, op):
         args = op.getarglist()
         base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), args)
-        result_loc = self.force_allocate_reg(op)
-        if sign:
-            sign_loc = imm1
+        scale_box = op.getarg(3)
+        offset_box = op.getarg(4)
+        size_box = op.getarg(5)
+        assert isinstance(scale_box, ConstInt)
+        assert isinstance(offset_box, ConstInt)
+        assert isinstance(size_box, ConstInt)
+        factor = scale_box.value
+        offset = offset_box.value
+        size = size_box.value
+        assert size >= 1
+        if size == 1:
+            need_lower_byte = True
         else:
-            sign_loc = imm0
-        self.perform(op, [base_loc, ofs_loc, size_loc, sign_loc], result_loc)
-
-    consider_getfield_gc_i = _consider_getfield
-    consider_getfield_gc_r = _consider_getfield
-    consider_getfield_gc_f = _consider_getfield
-    consider_getfield_raw_i = _consider_getfield
-    consider_getfield_raw_f = _consider_getfield
-    consider_getfield_gc_pure_i = _consider_getfield
-    consider_getfield_gc_pure_r = _consider_getfield
-    consider_getfield_gc_pure_f = _consider_getfield
+            need_lower_byte = False
+        value_loc = self.make_sure_var_in_reg(op.getarg(2), args,
+                                          need_lower_byte=need_lower_byte)
+        ofs_loc = self.rm.make_sure_var_in_reg(op.getarg(1), args)
+        self.perform_discard(op, [base_loc, ofs_loc, value_loc,
+                                  imm(factor), imm(offset), imm(size)])
 
     def consider_increment_debug_counter(self, op):
         base_loc = self.loc(op.getarg(0))
         self.perform_discard(op, [base_loc])
 
-    def _consider_getarrayitem(self, op):
-        itemsize, ofs, sign = unpack_arraydescr(op.getdescr())
+    def _consider_gc_load(self, op):
         args = op.getarglist()
         base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), args)
         ofs_loc = self.rm.make_sure_var_in_reg(op.getarg(1), args)
         result_loc = self.force_allocate_reg(op)
-        if sign:
+        size_box = op.getarg(2)
+        assert isinstance(size_box, ConstInt)
+        nsize = size_box.value      # negative for "signed"
+        size_loc = imm(abs(nsize))
+        if nsize < 0:
             sign_loc = imm1
         else:
             sign_loc = imm0
-        self.perform(op, [base_loc, ofs_loc, imm(itemsize), imm(ofs),
-                          sign_loc], result_loc)
+        self.perform(op, [base_loc, ofs_loc, size_loc, sign_loc], result_loc)
 
-    consider_getarrayitem_gc_i = _consider_getarrayitem
-    consider_getarrayitem_gc_r = _consider_getarrayitem
-    consider_getarrayitem_gc_f = _consider_getarrayitem
-    consider_getarrayitem_raw_i = _consider_getarrayitem
-    consider_getarrayitem_raw_f = _consider_getarrayitem
-    consider_getarrayitem_gc_pure_i = _consider_getarrayitem
-    consider_getarrayitem_gc_pure_r = _consider_getarrayitem
-    consider_getarrayitem_gc_pure_f = _consider_getarrayitem
-    consider_raw_load_i = _consider_getarrayitem
-    consider_raw_load_f = _consider_getarrayitem
+    consider_gc_load_i = _consider_gc_load
+    consider_gc_load_r = _consider_gc_load
+    consider_gc_load_f = _consider_gc_load
 
-    def _consider_getinteriorfield(self, op):
-        t = unpack_interiorfielddescr(op.getdescr())
-        ofs, itemsize, fieldsize, sign = imm(t[0]), imm(t[1]), imm(t[2]), t[3]
-        if sign:
-            sign_loc = imm1
-        else:
-            sign_loc = imm0
+    def _consider_gc_load_indexed(self, op):
         args = op.getarglist()
         base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), args)
-        index_loc = self.rm.make_sure_var_in_reg(op.getarg(1), args)
-        # 'base' and 'index' are put in two registers (or one if 'index'
-        # is an immediate).  'result' can be in the same register as
-        # 'index' but must be in a different register than 'base'.
-        result_loc = self.force_allocate_reg(op, [op.getarg(0)])
-        assert isinstance(result_loc, RegLoc)
-        # two cases: 1) if result_loc is a normal register, use it as temp_loc
-        if not result_loc.is_xmm:
-            temp_loc = result_loc
+        ofs_loc = self.rm.make_sure_var_in_reg(op.getarg(1), args)
+        result_loc = self.force_allocate_reg(op)
+        scale_box = op.getarg(2)
+        offset_box = op.getarg(3)
+        size_box = op.getarg(4)
+        assert isinstance(scale_box, ConstInt)
+        assert isinstance(offset_box, ConstInt)
+        assert isinstance(size_box, ConstInt)
+        scale = scale_box.value
+        offset = offset_box.value
+        nsize = size_box.value      # negative for "signed"
+        size_loc = imm(abs(nsize))
+        if nsize < 0:
+            sign_loc = imm1
         else:
-            # 2) if result_loc is an xmm register, we (likely) need another
-            # temp_loc that is a normal register.  It can be in the same
-            # register as 'index' but not 'base'.
-            tempvar = TempVar()
-            temp_loc = self.rm.force_allocate_reg(tempvar, [op.getarg(0)])
-            self.rm.possibly_free_var(tempvar)
-        self.perform(op, [base_loc, ofs, itemsize, fieldsize,
-                          index_loc, temp_loc, sign_loc], result_loc)
+            sign_loc = imm0
+        locs = [base_loc, ofs_loc, imm(scale), imm(offset), size_loc, sign_loc]
+        self.perform(op, locs, result_loc)
 
-    consider_getinteriorfield_gc_i = _consider_getinteriorfield
-    consider_getinteriorfield_gc_r = _consider_getinteriorfield
-    consider_getinteriorfield_gc_f = _consider_getinteriorfield
+    consider_gc_load_indexed_i = _consider_gc_load_indexed
+    consider_gc_load_indexed_r = _consider_gc_load_indexed
+    consider_gc_load_indexed_f = _consider_gc_load_indexed
 
     def consider_int_is_true(self, op):
         # doesn't need arg to be in a register
@@ -1227,32 +1145,6 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
         resloc = self.force_allocate_reg(op, [op.getarg(0)])
         self.perform(op, [argloc], resloc)
 
-    def consider_strlen(self, op):
-        args = op.getarglist()
-        base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), args)
-        result_loc = self.rm.force_allocate_reg(op)
-        self.perform(op, [base_loc], result_loc)
-
-    consider_unicodelen = consider_strlen
-
-    def consider_arraylen_gc(self, op):
-        arraydescr = op.getdescr()
-        assert isinstance(arraydescr, ArrayDescr)
-        ofs = arraydescr.lendescr.offset
-        args = op.getarglist()
-        base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), args)
-        result_loc = self.rm.force_allocate_reg(op)
-        self.perform(op, [base_loc, imm(ofs)], result_loc)
-
-    def consider_strgetitem(self, op):
-        args = op.getarglist()
-        base_loc = self.rm.make_sure_var_in_reg(op.getarg(0), args)
-        ofs_loc = self.rm.make_sure_var_in_reg(op.getarg(1), args)
-        result_loc = self.rm.force_allocate_reg(op)
-        self.perform(op, [base_loc, ofs_loc], result_loc)
-
-    consider_unicodegetitem = consider_strgetitem
-
     def consider_copystrcontent(self, op):
         self._consider_copystrcontent(op, is_unicode=False)
 
