@@ -624,6 +624,12 @@ def op_jit_record_exact_class(x, y):
 def op_jit_ffi_save_result(*args):
     pass
 
+def op_jit_enter_portal_frame(x):
+    pass
+
+def op_jit_leave_portal_frame():
+    pass
+
 def op_get_group_member(TYPE, grpptr, memberoffset):
     from rpython.rtyper.lltypesystem import llgroup
     assert isinstance(memberoffset, llgroup.GroupMemberOffset)
@@ -702,6 +708,17 @@ def op_raw_load(TVAL, p, ofs):
     return p[0]
 op_raw_load.need_result_type = True
 
+def op_gc_load_indexed(TVAL, p, index, scale, base_ofs):
+    # 'base_ofs' should be a CompositeOffset(..., ArrayItemsOffset).
+    # 'scale' should be a llmemory.sizeof().
+    from rpython.rtyper.lltypesystem import rffi
+    ofs = base_ofs + scale * index
+    if isinstance(ofs, int):
+        return op_raw_load(TVAL, p, ofs)
+    p = rffi.cast(rffi.CArrayPtr(TVAL), llmemory.cast_ptr_to_adr(p) + ofs)
+    return p[0]
+op_gc_load_indexed.need_result_type = True
+
 def op_likely(x):
     assert isinstance(x, bool)
     return x
